@@ -1,11 +1,13 @@
 # import a lib can handle http request
 import logging
 import os
+import stat
 import logging.handlers
 import http.client, urllib.parse
 from pickle import FALSE, NONE
 import shutil
 import datetime
+import re
 
 _logger = None
 
@@ -34,6 +36,7 @@ def setup_logging(logger_name, log_file_name, loglevel=logging.DEBUG, log_output
     """
 
     if not os.path.exists(log_output_dir):
+        # mkdir recursively
         os.makedirs(log_output_dir)
 
     #filename=f"{log_output_dir}/.hc_upgrade_tools/hc_upgrade_tools.log",
@@ -128,6 +131,8 @@ def handle_artifacts(source_file_path, target_dir, source_sub_folder=None):
       target_dir (str): 目标目录
       source_sub_folder (str): 指定只需要的交付物中的子目录
     """
+    logger = get_logger()
+
     if os.path.exists(target_dir):
         shutil.rmtree(target_dir)
 
@@ -152,28 +157,46 @@ def handle_artifacts(source_file_path, target_dir, source_sub_folder=None):
                 z.extractall(temp_target_dir)
                 # move all files in temp_target_dir/source_sub_dir to target_dir
                 for file in os.listdir(os.path.join(temp_target_dir, source_sub_folder)):
-                    # if filename is "wait_for_db_ready.sh", chmod +x first
-                    if file == "wait_for_db_ready.sh":
-                        os.chmod(os.path.join(temp_target_dir, source_sub_folder, file), 0o777)
-
                     os.rename(os.path.join(temp_target_dir, source_sub_folder, file), os.path.join(target_dir, file))
+
                 # remove temp_target_dir all
                 shutil.rmtree(temp_target_dir)
 
+                # walk file named  "wait_for_db_ready.sh" nested in target_dir
+                for root, dirs, files in os.walk(target_dir):
+                    for file in files:
+                        if file == "wait_for_db_ready.sh":
+                            logger.info(f"find wait_for_db_ready.sh in {root}")
+                            st = os.stat(os.path.join(root, file))
+                            os.chmod(os.path.join(root, file), st.st_mode | stat.S_IEXEC)
+
     except Exception as error:
         print(f"unzip source file failed, error is {error}")
         return FALSE
 
-    _logger.info(f"handle artifacts success, target dir is {target_dir}")
+    logger.info(f"handle artifacts success, target dir is {target_dir}")
     return True
 
 
-def create_new_release_dir(release_root_dir):
+def create_new_release_dir(release_root_dir, max_history_release_count=5):
     # release dir like: 20210514175142
     release_dir = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
     release_dir = os.path.join(release_root_dir, release_dir)
     if not os.path.exists(release_dir):
         os.makedirs(release_dir)
+
+    # sort release dir name match like 20210514175142 and make sure release dir is a dir
+    # keep latest max_history_release_count release dir
+    release_dir_list = [os.path.join(release_root_dir, release_dir_name) for release_dir_name in os.listdir(release_root_dir) if
+                    os.path.isdir(os.path.join(release_root_dir, release_dir_name)) and
+                    re.match(r"\d{14}", release_dir_name)]
+
+    release_dir_list = sorted(release_dir_list, key=lambda x: os.path.getmtime(x), reverse=True)
+
+    if len(release_dir_list) > max_history_release_count:
+        for dir in release_dir_list[max_history_release_count:]:
+            shutil.rmtree(dir)
+
     return release_dir
 
 
